@@ -126,8 +126,8 @@ where
                 }
                 if self
                     .data
-                    .as_ref()
-                    .and_then(|data| data.vars.get(&i.to_id()))
+                    .vars
+                    .get(&i.to_id())
                     .map(|v| v.assign_count == 0 && !v.declared_as_fn_param)
                     .unwrap_or(false)
                 {
