@@ -3,7 +3,7 @@ use swc_atoms::{js_word, JsWord};
 use swc_common::util::take::Take;
 use swc_ecma_utils::Id;
 
-use crate::pass::{compute_char_freq::CharFreqInfo, mangle_names::rename_map::RenameMap};
+use crate::{pass::mangle_names::rename_map::RenameMap, util::base54};
 
 #[derive(Debug, Default)]
 pub(crate) struct Scope {
@@ -14,6 +14,10 @@ pub(crate) struct Scope {
 
 #[derive(Debug, Default)]
 pub struct ScopeData {
+    /// This is add-only.
+    ///
+    /// If the add-only contraint is violated, it is very likely to be a bug,
+    /// because we merge every items in children to current scope.
     all: FxHashSet<Id>,
 
     queue: Vec<(Id, u32)>,
@@ -49,9 +53,17 @@ impl Scope {
         self.data.all.insert(id.clone());
     }
 
+    /// Copy `children.data.all` to `self.data.all`.
+    pub(super) fn prepare_renaming(&mut self) {
+        self.children.iter_mut().for_each(|child| {
+            child.prepare_renaming();
+
+            self.data.all.extend(child.data.all.iter().cloned());
+        });
+    }
+
     pub(super) fn rename(
         &mut self,
-        f: &CharFreqInfo,
         to: &mut RenameMap,
         preserved: &FxHashSet<Id>,
         preserved_symbols: &FxHashSet<JsWord>,
@@ -66,10 +78,11 @@ impl Scope {
             }
 
             loop {
-                let (_, sym) = f.incr_base54(&mut n);
+                let sym = base54::encode(&mut n, true);
 
                 let sym: JsWord = sym.into();
 
+                // TODO: Use base54::decode
                 if preserved_symbols.contains(&sym) {
                     continue;
                 }
@@ -78,7 +91,6 @@ impl Scope {
                     to.insert(id.clone(), sym);
                     // self.data.decls.remove(&id);
                     // self.data.usages.remove(&id);
-                    self.data.all.remove(&id);
 
                     break;
                 }
@@ -86,7 +98,7 @@ impl Scope {
         }
 
         for child in self.children.iter_mut() {
-            child.rename(f, to, preserved, preserved_symbols);
+            child.rename(to, preserved, preserved_symbols);
         }
     }
 
@@ -109,8 +121,6 @@ impl Scope {
             }
         }
 
-        self.children
-            .iter()
-            .all(|c| c.can_rename(id, symbol, renamed))
+        true
     }
 }
